/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.scheduler.NotificationSeverity',
    'QDesk.model.message.channel.MessageChannel',
    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzExecNotificationRuleStore',

    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzExecNotificationRulesGrid',

  cls : 'quartzExecNotificationRulesGrid',

  scrollable : true,

  //border : false,


  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
        '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
        '<tpl if="!description">{[QDesk.Msg.get( \"QDesk.notAvailable\" )]}</tpl>' +
        '</p>'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var notificationRuleStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecNotificationRuleStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addNotificationRuleButton',
          scale : 'small',
          iconCls : 'addNotificationRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipAddExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ],
          qdNotAuthorizedIconCls : 'addNotificationRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'addNotificationRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'deleteNotificationRuleButton',
          scale : 'small',
          iconCls : 'deleteNotificationRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipDeleteExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_DELETE ],
          qdNotAuthorizedIconCls : 'deleteNotificationRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'deleteNotificationRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'editNotificationRuleButton',
          scale : 'small',
          iconCls : 'editNotificationRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEditExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'editNotificationRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'editNotificationRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'cloneNotificationRuleButton',
          scale : 'small',
          iconCls : 'cloneNotificationRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipCloneExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ],
          qdNotAuthorizedIconCls : 'cloneNotificationRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                this.fireEvent( 'cloneNotificationRule' );
              }
            }
          }
        },
        '-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextFieldFilterQueryText' ),
          store : notificationRuleStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],


      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.3
        },

        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'severityColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSeverityColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSeverityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSeverityColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var iconClass, tooltipText;
            var severity = record.get( 'severity' );

            if ( severity == QDesk.model.scheduler.NotificationSeverity.INFO )
              iconClass = 'notificationSeverityInfo nonClickable';
            else if ( severity == QDesk.model.scheduler.NotificationSeverity.WARNING )
              iconClass = 'notificationSeverityWarning nonClickable';
            else if ( severity == QDesk.model.scheduler.NotificationSeverity.ERROR )
              iconClass = 'notificationSeverityError nonClickable';
            else if ( severity == QDesk.model.scheduler.NotificationSeverity.FATAL )
              iconClass = 'notificationSeverityFatal nonClickable';
            else
              Ext.Error.raise( 'Unexpected severity value: ' + value );

            tooltipText = QDesk.util.EnumUtils.getNotificationSeverityName( severity );

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>';
          }
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'enabledColumn',
          align : 'center',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledColumn' ),
          dataIndex : 'enabled',
          sortable : false,
          groupable : false,
          width : 100
        },

        {
          itemId : 'enabledOnWeekdaysColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledOnWeekdaysColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledOnWeekdaysColumn' ),
          dataIndex : 'enabledOnWeekday',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enums value */
          renderer : function ( value, metadata, record )
          {
            var result = '';
            Ext.each( value, function ( weekday, index, weekdays )
            {
              result += QDesk.util.EnumUtils.getWeekdayShortName( weekday );
              if ( index < weekdays.length - 1 )
                result += ', ';
            } );

            return result;
          },
          flex : 1
        },

        {
          itemId : 'enabledFromTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledFromTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledFromTimeColumn' ),
          dataIndex : 'enabledFromTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'enabledToTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledToTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledToTimeColumn' ),
          dataIndex : 'enabledToTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },

        {
          itemId : 'conditionTypeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textConditionTypeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipConditionTypeColumn' ),
          dataIndex : 'conditionType',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enum value */
          renderer : function ( value, metadata, record )
          {
            switch ( value )
            {
              case QDesk.model.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS:
                var conditionExecStatus = record.get( 'conditionExecStatus' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExecStatusName( conditionExecStatus ) + ']';
                break;

              case QDesk.model.scheduler.ExecNotificationRuleConditionType.MAX_DURATION:
                var conditionMaxDuration = record.get( 'conditionMaxDuration' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    conditionMaxDuration + 'ms]';
                break;

              case QDesk.model.scheduler.ExecNotificationRuleConditionType.EXPRESSION:
                var conditionExpression = record.get( 'conditionExpression' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExpressionLanguageName( conditionExpression.language ) + ']';
                break;

              default:
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value );
            }
          },
          flex : 0.7
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'messageChannelProfileColumn',
          dataIndex : 'messageChannelProfileName',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessageChannelProfileColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessageChannelProfileColumn' ),
          sortable : false,
          groupable : false,
          flex : 1,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metadata, record )
          {
            var iconClass, tooltipText;

            if ( value )
            {
              var messageChannel = record.get( 'messageChannelProfileChannel' );
              tooltipText = QDesk.util.EnumUtils.getMessageChannelName( messageChannel );

              if ( messageChannel == QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT )
                iconClass = 'messageChannelCollaborationHipChat';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK )
                iconClass = 'messageChannelCollaborationSlack';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.EMAIL )
                iconClass = 'messageChannelEmail';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_AIM )
                iconClass = 'messageChannelImAim';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_GTALK )
                iconClass = 'messageChannelImGtalk';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_ICQ )
                iconClass = 'messageChannelImIcq';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_XMPP )
                iconClass = 'messageChannelImXmpp';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_YAHOO )
                iconClass = 'messageChannelImYahoo';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL )
                iconClass = 'messageChannelSmsClickatellCentral';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM )
                iconClass = 'messageChannelSmsClickatellPlatform';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK )
                iconClass = 'messageChannelSmsClockwork';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_NEXMO )
                iconClass = 'messageChannelSmsNexmo';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL )
                iconClass = 'messageChannelSmsSmsGlobal';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_TWILIO )
                iconClass = 'messageChannelSmsTwilio';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP )
                iconClass = 'messageChannelSnmpV1Trap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP )
                iconClass = 'messageChannelSnmpV2cTrap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP )
                iconClass = 'messageChannelSnmpV3Trap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.WEB_SERVICE )
                iconClass = 'messageChannelWebService';
            }
            else
            {
              value = QDesk.Msg.get( 'QDesk.notAvailable' );
              iconClass = 'messageChannelUnknown';
              var messageChannelProfileId = record.get( 'messageChannelProfileId' );
              tooltipText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessageChannelProfileNotFound',
                      messageChannelProfileId );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>' +
                '<span style="padding-left: 5px;">' + value + '</span>';
          }
        },

        {
          xtype : 'datecolumn',
          itemId : 'createdAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textCreatedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipCreatedAtColumn' ),
          dataIndex : 'createdAt',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'updatedAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textUpdatedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipUpdatedAtColumn' ),
          dataIndex : 'updatedAt',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'actioncolumn',
          itemId : 'messagesColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessagesColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessagesColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessagesColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/message-queue-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessagesColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var notifRuleId = record.get( 'id' );
                this.fireEvent( 'requestViewMessages', notifRuleId );
              }
            }
          ]
        }
      ],

      store : notificationRuleStore,

      qdesk : {
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );
  },


  /**
   * Invoked to create a context menu for the specified execution notification rule.
   * @param execNotifRule an execution notification rule.
   * @private
   */
  _createRowCtxMenu : function ( execNotifRule )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // enable rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuEnable' ),
          iconCls : 'enableExecNotificationRule',
          disabled : execNotifRule.get( 'enabled' ),
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'enableExecNotificationRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'enableRule', execNotifRule );
          }
        },
        // disable rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuDisable' ),
          iconCls : 'disableExecNotificationRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'disableExecNotificationRuleNAUTH',
          disabled : !execNotifRule.get( 'enabled' ),
          handler : function ()
          {
            me.fireEvent( 'disableRule', execNotifRule );
          }
        },
        '-',
        // delete rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuDelete' ),
          iconCls : 'deleteExecNotificationRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_DELETE ],
          qdNotAuthorizedIconCls : 'deleteExecNotificationRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteRule', execNotifRule );
          }
        },
        // edit rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuEdit' ),
          iconCls : 'editExecNotificationRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'editExecNotificationRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'editRule', execNotifRule );
          }
        },
        // clone rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuClone' ),
          iconCls : 'cloneExecNotificationRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ],
          qdNotAuthorizedIconCls : 'cloneExecNotificationRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'cloneRule', execNotifRule );
          }
        }
      ]
    } );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          var jobsTab = me.up( '#jobsTab' );
          var triggersTab = me.up( '#triggersTab' );

          // default empty text
          var emptyText = QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForScheduler' );

          if ( jobsTab )
          {
            var selectedJobs = jobsTab.down( '#jobsGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedJobs ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoJobSelected' );
            }
            else
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForJob' );
            }
          }
          else if ( triggersTab )
          {
            var selectedTriggers = triggersTab.down( '#triggersGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedTriggers ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoTriggerSelected' );
            }
            else
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForTrigger' );
            }
          }

          view.emptyText = emptyText;

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  }
} );
